#include <avr/io.h>      
#include <string.h>
#include <avr/interrupt.h>   // Interrupts and timers
#include <util/delay.h>      // Blocking delay functions
#include <avr/pgmspace.h>    // So we can store the 'font table' in ROM
#include <avr/eeprom.h>      // Date/time/pref backup in permanent EEPROM
#include <avr/wdt.h>     // Watchdog timer to repair lockups

#include "files/header/icecubeclock.h"
#include "files/header/language.h"
#include "files/header/sprueche.h"
#include "files/header/fonttable.h"
//Only necessary for debug via UART
//#include "files/debug_uart/util_uart.h"

//external c-files

#include "files/util.c"
#include "files/interrupt.c"
#include "files/init.c"
#include "files/display.c"
#include "files/menu.c"
#include "files/ambilight.c"
//Only necessary for debug via UART
//#include "files/debug_uart/util_uart.c"

/**************************** MAIN APP *****************************/

int main(void) {
  //  uint8_t i;
  uint8_t mcustate;

  // turn boost off
  TCCR0B = 0;
  BOOST_DDR |= _BV(BOOST);
  BOOST_PORT &= ~_BV(BOOST); // pull boost fet low

  // check if we were reset
  mcustate = MCUSR;
  MCUSR = 0;

  wdt_disable();
  // now turn it back on... 8 second time out
  //WDTCSR |= _BV(WDP0) | _BV(WDP1) | _BV(WDP2);
  //WDTCSR = _BV(WDE);
  wdt_enable(WDTO_2S);
  kickthedog();

  // we lost power at some point so lets alert the user
  // that the time may be wrong (the clock still works)
  ctrl_struct.timeunknown = 1;
  
  // we lost power at some point so rest the countdown
  countdown = 0;
  ctrl_struct.countdown_is_set = 0;
  ctrl_struct.countdown_alarming = 0;

  // have we read the time & date from eeprom?
  ctrl_struct.restored = 0;

  // setup uart
  //uart_init(BRRL_192);

  // set up analog comparator
  ACSR = _BV(ACBG) | _BV(ACIE); // use bandgap, intr. on toggle!
  _delay_ms(1);
  // settle!
  if (ACSR & _BV(ACO)) {
    // hmm we should not interrupt here
    ACSR |= _BV(ACI);

    // even in low power mode, we run the clock 

    clock_init();  

  } else {
    	// we aren't in low power mode so init stuff

    	// init io's
    	initbuttons();
    
    	VFDSWITCH_PORT &= ~_BV(VFDSWITCH);
    
   	 // set up button interrupts
    	// set off an interrupt if alarm is set or unset
    	EICRA = _BV(ISC00);
    	EIMSK = _BV(INT0);
  
    	displaymode = NONE;
    	vfd_init();
    
    	boost_init(eeprom_read_byte((uint8_t *)EE_BRIGHT));
    	sei();

    	time.r = eeprom_read_byte((uint8_t *)EE_REGION);
	effect_ctrl.effect = eeprom_read_byte((uint8_t *)EE_EFFECT);    
	effect_ctrl.ticktack = eeprom_read_byte((uint8_t *)EE_TICKTACK);
	effect_ctrl.light = eeprom_read_byte((uint8_t *)EE_LIGHT);

    	clock_init();  

	if (effect_ctrl.light == TRUE )
		PORTC |= _BV(LED2);
	else
		recreate_pwm_sequence();

    	speaker_init();

    	beep(5000, 1);
	beep(8000, 1);
	beep(4000, 1);
	
	displaymode = NONE;

    	display_str("tobi uhr", TRUE);
    	delayms(2000);

    	setalarmstate();
    	delayms(1000);

    	displaymode = SHOW_TIME;
  	}

while (1) {
    //_delay_ms(100);
    kickthedog();
    //uart_putc_hex(ACSR);
    if (ACSR & _BV(ACO)) {
      	gotosleep();
      	continue;
    }
    if (just_pressed & 0x1) {
      	just_pressed = 0;
	show_menu();
    } else if ((just_pressed & 0x2) || (just_pressed & 0x4)) {
      	just_pressed = 0;
      	displaymode = NONE;

	if (ctrl_struct.countdown_is_set)
		display_countdown(countdown, FALSE);
	else	
      		display_date(time.d, time.mo, time.y, DAY, FALSE);

      	kickthedog();
      	delayms(1500);
      	kickthedog();

      	displaymode = SHOW_TIME;     
    	} 
  }
}


